/******************************************************************************/
/*			  PhysTech, Inc - A332765.c			      */
/*	Copyright (C) Feb.21, 2020 Sergei Viznyuk <phystech@hotmail.com>      */
/*									      */
/*	This code is provided without warranty, without even the	      */
/*  implied warranty of merchantability or fitness for a particular purpose.  */
/*******************************************************************************

A332765.c

Prints prime number triplets and corresponding indices
per OEIS sequence A332765 and A332766

Usage: A332765 [-o output_file] [<max>]

*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>
#include <ctype.h>

#ifndef ULLONG_MAX
#define ULLONG_MAX   18446744073709551615ULL
#endif
/******************************************************************************/
int main(argn,argc)
int argn;
char *argc[];
{
  FILE *f=stdout;
  unsigned char *c;
  unsigned long long primes[10000],i=1ULL,p=4ULL,min=7ULL,max=104729ULL;
  register unsigned long long m,j;

  while ( (c=argc[i++]) )
    if ( *c=='-' )
      if ( *++c=='o' )
	if ( !(f=fopen(argc[i],"w")) )
	  { fprintf(stderr,"%s\n",strerror(errno)); return errno; }
	else i++;
      else goto er;
    else if ( isdigit(*c) ) max=strtoull(c,NULL,0); else goto er;
  if ( max < min ) goto er;

  primes[0]=2;
  primes[1]=3;
  primes[2]=5;
  primes[3]=7; 
  fprintf(f, "OEIS A332765\tOEIS A332766\n"
	"Prime numbers\tCorresponding indices\n"
	"-------------------------------------\n"
	"5,3,2,0\t\t3,2,1\n7,5,2,0\t\t4,3,1\n");

  while ( (min+=2ULL) <= max )
    if ( !((min+1ULL)%6ULL)  || !((min-1ULL)%6ULL) )
      { for (i=3ULL;i*i<min;i+=2) if ( ! (min%i) ) break;
        if ( i*i > min )
	  { primes[p]=min;
	    for (i=p/2;i<p;i++)
	      { j=min-primes[i]-primes[p-i-1];
		if ( j%2ULL )
		  { for (m=3ULL;m*m<j;m+=2) if ( ! (j%m) ) break;
		    if ( m*m > j )
		      fprintf(f,"%llu,%llu,%llu,%llu\t%llu,%llu,%llu\n",
			min,primes[i],primes[p-i-1],j,p+1,i+1,p-i);
		  }
		else if ( j==2 || j==0 )
		  fprintf(f,"%llu,%llu,%llu,%llu\t%llu,%llu,%llu\n",
			min,primes[i],primes[p-i-1],j,p+1,i+1,p-i);
	      }
	    p++;
	  }
      }
  return 0;
er:
  fprintf(stderr,"Usage: A332765 [-o output_file] [<max>]\n");
  return 1;
}
